import { DataSource } from '@angular/cdk/table';

import { BehaviorSubject, Observable, merge, of } from 'rxjs';
import { map } from 'rxjs/operators';

import { CollectionViewer } from '@angular/cdk/collections';

import { BetHistory } from 'src/modules/game/model/bet-history.model';
import { BetHistoryService } from 'src/modules/game/service/bet-historyservice';

import { FormControl, FormGroup, Validators, FormArray } from '@angular/forms';

const ELEMENT_DATA: BetHistory[] = [
  {
    id: 1,
    thumbNail: '/images/thumbname/1.jpg',
    title: '오픈기념 이벤트',
    imageWidth: 200,
    imageHeight: 200,
    imageX: 100,
    imageY: 100,
    openStatus: true,
    updatedAt: null,
    createdAt: null
  },
  {
    id: 2,
    thumbNail: '/images/thumbname/2.jpg',
    title: '첫충전 이벤트',
    imageWidth: 200,
    imageHeight: 200,
    imageX: 100,
    imageY: 100,
    openStatus: true,
    updatedAt: null,
    createdAt: null
  },
  {
    id: 3,
    thumbNail: '/images/thumbname/3.jpg',
    title: '친구초대 이벤트',
    imageWidth: 200,
    imageHeight: 200,
    imageX: 100,
    imageY: 100,
    openStatus: true,
    updatedAt: null,
    createdAt: null
  }
];

export class BetHistoryDataSource extends DataSource<BetHistory> {
  private _objectStore: BetHistory[] = [];
  private _ObjectsSubject$ = new BehaviorSubject<BetHistory[]>([]);
  private _loadingSubject$ = new BehaviorSubject<boolean>(false);

  public loading$ = this._loadingSubject$.asObservable();
  public formg: FormGroup;

  constructor(private betHistoryService: BetHistoryService) {
    super();
  }

  connect(collectionViewer: CollectionViewer): Observable<BetHistory[]> {
    // return this.fixBonusService.getAll().pipe(
    //   map(res => {
    //     res.forEach(m => this._objectStore.push(m as FixBonus));
    //     this._ObjectsSubject$.next(this._objectStore);
    //     let fa = <FormArray>this.formg.get('formarray');
    //     res.forEach(r => fa.push(this.createRowFormGroup(r)));
    //     return res;
    //   })
    // );
    return of(ELEMENT_DATA).pipe(
      map(res => {
        res.forEach(m => this._objectStore.push(m as BetHistory));
        this._ObjectsSubject$.next(this._objectStore);
        let fa = <FormArray>this.formg.get('formarray');
        res.forEach(r => fa.push(this.createRowFormGroup(r)));
        return res;
      })
    );
  }

  // id: 3,
  // thumbNail: '/images/thumbname/3.jpg',
  // title: '친구초대 이벤트',
  // imageWidth: 200,
  // imageHeight: 200,
  // imageX: 100,
  // imageY: 100,
  // openStatus: true,
  // updatedAt: null,
  // createdAt: null
  createRowFormGroup(r: BetHistory): FormGroup {
    let f = new FormGroup({
      title: this.createNewFormContorl(r, 'title'),
      imageWidth: this.createNewFormContorl(r, 'imageWidth'),
      imageHeight: this.createNewFormContorl(r, 'imageHeight'),
      imageX: this.createNewFormContorl(r, 'imageX'),
      imageY: this.createNewFormContorl(r, 'imageY'),
      openStatus: this.createNewFormContorl(r, 'openStatus')
    });
    return f;
  }

  createNewFormContorl(r: BetHistory, propName: string): FormControl {
    let m = new FormControl(r[propName], Validators.required);
    m.valueChanges.subscribe(val => {
      r[propName] = val;
    });
    return m;
  }

  disconnect(collectionViewer: CollectionViewer): void {
    this._ObjectsSubject$.complete();
    this._loadingSubject$.complete();
  }
}
