import { DataSource } from '@angular/cdk/table';

import { BehaviorSubject, Observable, merge, of } from 'rxjs';
import { map } from 'rxjs/operators';

import { CollectionViewer } from '@angular/cdk/collections';

import { RouletteCoupon } from 'src/modules/special/roulette-coupon/model/roulette-coupon.model';
import { RouletteCouponService } from 'src/modules/special/roulette-coupon/service/roulette-coupon.service';

import { FormControl, FormGroup, Validators, FormArray } from '@angular/forms';

const ELEMENT_DATA: RouletteCoupon[] = [
  {
    id: 1,
    price: '10,000',
    rate: '5',
    priceWon: '10,000원',
    createdAt: null,
    updatedAt: null
  },
  {
    id: 2,
    price: '5,000',
    rate: '20',
    priceWon: '5,000원',
    createdAt: null,
    updatedAt: null
  },
  {
    id: 3,
    price: '100,000',
    rate: '0',
    priceWon: '100,000원',
    createdAt: null,
    updatedAt: null
  },
  {
    id: 4,
    price: '10,000',
    rate: '5',
    priceWon: '10,000원',
    createdAt: null,
    updatedAt: null
  },
  {
    id: 5,
    price: '5,000',
    rate: '20',
    priceWon: '5,000원',
    createdAt: null,
    updatedAt: null
  },
  {
    id: 6,
    price: '300,000',
    rate: '0',
    priceWon: '300,000원',
    createdAt: null,
    updatedAt: null
  },
  {
    id: 7,
    price: '10,000',
    rate: '5',
    priceWon: '10,000원',
    createdAt: null,
    updatedAt: null
  }
];

export class RouletteCouponDataSource extends DataSource<RouletteCoupon> {
  private _objectStore: RouletteCoupon[] = [];
  private _ObjectsSubject$ = new BehaviorSubject<RouletteCoupon[]>([]);
  private _loadingSubject$ = new BehaviorSubject<boolean>(false);

  public loading$ = this._loadingSubject$.asObservable();
  public formg: FormGroup;

  constructor(private RouletteCouponService: RouletteCouponService) {
    super();
  }

  connect(collectionViewer: CollectionViewer): Observable<RouletteCoupon[]> {
    // return this.RouletteCouponService.getAll().pipe(
    //   map(res => {
    //     res.forEach(m => this._objectStore.push(m as RouletteCoupon));
    //     this._ObjectsSubject$.next(this._objectStore);
    //     let fa = <FormArray>this.formg.get('formarray');
    //     res.forEach(r => fa.push(this.createRowFormGroup(r)));
    //     return res;
    //   })
    // );
    return of(ELEMENT_DATA).pipe(
      map(res => {
        res.forEach(m => this._objectStore.push(m as RouletteCoupon));
        this._ObjectsSubject$.next(this._objectStore);
        let fa = <FormArray>this.formg.get('formarray');
        res.forEach(r => fa.push(this.createRowFormGroup(r)));
        return res;
      })
    );
  }

  createRowFormGroup(r: RouletteCoupon): FormGroup {
    let f = new FormGroup({
      id: this.createNewFormContorl(r, 'id'),
      price: this.createNewFormContorl(r, 'price'),
      rate: this.createNewFormContorl(r, 'rate'),
      priceWon: this.createNewFormContorl(r, 'priceWon')
    });
    return f;
  }

  createNewFormContorl(r: RouletteCoupon, propName: string): FormControl {
    let m = new FormControl(r[propName], Validators.required);
    m.valueChanges.subscribe(val => {
      r[propName] = val;
    });
    return m;
  }

  disconnect(collectionViewer: CollectionViewer): void {
    this._ObjectsSubject$.complete();
    this._loadingSubject$.complete();
  }
}
